@@ -2,12 +2,10 @@ package engine
 
 import (
 	"bytes"
-	"io/ioutil"
-	"os"
-	"path"
-	"path/filepath"
 	"strings"
 	"testing"
+
+	"github.com/docker/docker/pkg/ioutils"
 )
 
 func TestRegister(t *testing.T) {
@@ -21,7 +19,7 @@ func TestRegister(t *testing.T) {
 	// Register is global so let's cleanup to avoid conflicts
 	defer unregister("dummy1")
 
-	eng := newTestEngine(t)
+	eng := New()
 
 	//Should fail because global handlers are copied
 	//at the engine creation
@@ -40,7 +38,7 @@ func TestRegister(t *testing.T) {
 }
 
 func TestJob(t *testing.T) {
-	eng := newTestEngine(t)
+	eng := New()
 	job1 := eng.Job("dummy1", "--level=awesome")
 
 	if job1.handler != nil {
@@ -65,9 +63,19 @@ func TestJob(t *testing.T) {
 	}
 }
 
+func TestEngineShutdown(t *testing.T) {
+	eng := New()
+	if eng.IsShutdown() {
+		t.Fatalf("Engine should not show as shutdown")
+	}
+	eng.Shutdown()
+	if !eng.IsShutdown() {
+		t.Fatalf("Engine should show as shutdown")
+	}
+}
+
 func TestEngineCommands(t *testing.T) {
-	eng := newTestEngine(t)
-	defer os.RemoveAll(eng.Root())
+	eng := New()
 	handler := func(job *Job) Status { return StatusOK }
 	eng.Register("foo", handler)
 	eng.Register("bar", handler)
@@ -83,44 +91,9 @@ func TestEngineCommands(t *testing.T) {
 	}
 }
 
-func TestEngineRoot(t *testing.T) {
-	tmp, err := ioutil.TempDir("", "docker-test-TestEngineCreateDir")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tmp)
-	// We expect Root to resolve to an absolute path.
-	// FIXME: this should not be necessary.
-	// Until the above FIXME is implemented, let's check for the
-	// current behavior.
-	tmp, err = filepath.EvalSymlinks(tmp)
-	if err != nil {
-		t.Fatal(err)
-	}
-	tmp, err = filepath.Abs(tmp)
-	if err != nil {
-		t.Fatal(err)
-	}
-	dir := path.Join(tmp, "dir")
-	eng, err := New(dir)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if st, err := os.Stat(dir); err != nil {
-		t.Fatal(err)
-	} else if !st.IsDir() {
-		t.Fatalf("engine.New() created something other than a directory at %s", dir)
-	}
-	if r := eng.Root(); r != dir {
-		t.Fatalf("Expected: %v\nReceived: %v", dir, r)
-	}
-}
-
 func TestEngineString(t *testing.T) {
-	eng1 := newTestEngine(t)
-	defer os.RemoveAll(eng1.Root())
-	eng2 := newTestEngine(t)
-	defer os.RemoveAll(eng2.Root())
+	eng1 := New()
+	eng2 := New()
 	s1 := eng1.String()
 	s2 := eng2.String()
 	if eng1 == eng2 {
@@ -128,20 +101,8 @@ func TestEngineString(t *testing.T) {
 	}
 }
 
-func TestEngineLogf(t *testing.T) {
-	eng := newTestEngine(t)
-	defer os.RemoveAll(eng.Root())
-	input := "Test log line"
-	if n, err := eng.Logf("%s\n", input); err != nil {
-		t.Fatal(err)
-	} else if n < len(input) {
-		t.Fatalf("Test: Logf() should print at least as much as the input\ninput=%d\nprinted=%d", len(input), n)
-	}
-}
-
 func TestParseJob(t *testing.T) {
-	eng := newTestEngine(t)
-	defer os.RemoveAll(eng.Root())
+	eng := New()
 	// Verify that the resulting job calls to the right place
 	var called bool
 	eng.Register("echo", func(job *Job) Status {
@@ -175,3 +136,101 @@ func TestParseJob(t *testing.T) {
 		t.Fatalf("Job was not called")
 	}
 }
+
+func TestCatchallEmptyName(t *testing.T) {
+	eng := New()
+	var called bool
+	eng.RegisterCatchall(func(job *Job) Status {
+		called = true
+		return StatusOK
+	})
+	err := eng.Job("").Run()
+	if err == nil {
+		t.Fatalf("Engine.Job(\"\").Run() should return an error")
+	}
+	if called {
+		t.Fatalf("Engine.Job(\"\").Run() should return an error")
+	}
+}
+
+// Ensure that a job within a job both using the same underlying standard
+// output writer does not close the output of the outer job when the inner
+// job's stdout is wrapped with a NopCloser. When not wrapped, it should
+// close the outer job's output.
+func TestNestedJobSharedOutput(t *testing.T) {
+	var (
+		outerHandler Handler
+		innerHandler Handler
+		wrapOutput   bool
+	)
+
+	outerHandler = func(job *Job) Status {
+		job.Stdout.Write([]byte("outer1"))
+
+		innerJob := job.Eng.Job("innerJob")
+
+		if wrapOutput {
+			innerJob.Stdout.Add(ioutils.NopWriteCloser(job.Stdout))
+		} else {
+			innerJob.Stdout.Add(job.Stdout)
+		}
+
+		if err := innerJob.Run(); err != nil {
+			t.Fatal(err)
+		}
+
+		// If wrapOutput was *false* this write will do nothing.
+		// FIXME (jlhawn): It should cause an error to write to
+		// closed output.
+		job.Stdout.Write([]byte(" outer2"))
+
+		return StatusOK
+	}
+
+	innerHandler = func(job *Job) Status {
+		job.Stdout.Write([]byte(" inner"))
+
+		return StatusOK
+	}
+
+	eng := New()
+	eng.Register("outerJob", outerHandler)
+	eng.Register("innerJob", innerHandler)
+
+	// wrapOutput starts *false* so the expected
+	// output of running the outer job will be:
+	//
+	//     "outer1 inner"
+	//
+	outBuf := new(bytes.Buffer)
+	outerJob := eng.Job("outerJob")
+	outerJob.Stdout.Add(outBuf)
+
+	if err := outerJob.Run(); err != nil {
+		t.Fatal(err)
+	}
+
+	expectedOutput := "outer1 inner"
+	if outBuf.String() != expectedOutput {
+		t.Fatalf("expected job output to be %q, got %q", expectedOutput, outBuf.String())
+	}
+
+	// Set wrapOutput to true so that the expected
+	// output of running the outer job will be:
+	//
+	//     "outer1 inner outer2"
+	//
+	wrapOutput = true
+	outBuf.Reset()
+	outerJob = eng.Job("outerJob")
+	outerJob.Stdout.Add(outBuf)
+
+	if err := outerJob.Run(); err != nil {
+		t.Fatal(err)
+	}
+
+	expectedOutput = "outer1 inner outer2"
+	if outBuf.String() != expectedOutput {
+		t.Fatalf("expected job output to be %q, got %q", expectedOutput, outBuf.String())
+	}
+}
