@@ -1,11 +1,14 @@
 package engine
 
 import (
+	"bytes"
 	"fmt"
 	"io"
-	"os"
 	"strings"
+	"sync"
 	"time"
+
+	log "github.com/Sirupsen/logrus"
 )
 
 // A job is the fundamental unit of work in the docker engine.
@@ -17,8 +20,7 @@ import (
 // environment variables, standard streams for input, output and error, and
 // an exit status which can indicate success (0) or error (anything else).
 //
-// One slight variation is that jobs report their status as a string. The
-// string "0" indicates success, and any other strings indicates an error.
+// For status, 0 indicates success, and any other integers indicates an error.
 // This allows for richer error reporting.
 //
 type Job struct {
@@ -32,7 +34,13 @@ type Job struct {
 	handler Handler
 	status  Status
 	end     time.Time
-	onExit  []func()
+	closeIO bool
+
+	// When closed, the job has been cancelled.
+	// Note: not all jobs implement cancellation.
+	// See Job.Cancel() and Job.WaitCancelled()
+	cancelled  chan struct{}
+	cancelOnce sync.Once
 }
 
 type Status int
@@ -47,18 +55,34 @@ const (
 // If the job returns a failure status, an error is returned
 // which includes the status.
 func (job *Job) Run() error {
+	if job.Eng.IsShutdown() && !job.GetenvBool("overrideShutdown") {
+		return fmt.Errorf("engine is shutdown")
+	}
+	// FIXME: this is a temporary workaround to avoid Engine.Shutdown
+	// waiting 5 seconds for server/api.ServeApi to complete (which it never will)
+	// everytime the daemon is cleanly restarted.
+	// The permanent fix is to implement Job.Stop and Job.OnStop so that
+	// ServeApi can cooperate and terminate cleanly.
+	if job.Name != "serveapi" {
+		job.Eng.l.Lock()
+		job.Eng.tasks.Add(1)
+		job.Eng.l.Unlock()
+		defer job.Eng.tasks.Done()
+	}
 	// FIXME: make this thread-safe
 	// FIXME: implement wait
 	if !job.end.IsZero() {
 		return fmt.Errorf("%s: job has already completed", job.Name)
 	}
 	// Log beginning and end of the job
-	job.Eng.Logf("+job %s", job.CallString())
-	defer func() {
-		job.Eng.Logf("-job %s%s", job.CallString(), job.StatusString())
-	}()
-	var errorMessage string
-	job.Stderr.AddString(&errorMessage)
+	if job.Eng.Logging {
+		log.Infof("+job %s", job.CallString())
+		defer func() {
+			log.Infof("-job %s%s", job.CallString(), job.StatusString())
+		}()
+	}
+	var errorMessage = bytes.NewBuffer(nil)
+	job.Stderr.Add(errorMessage)
 	if job.handler == nil {
 		job.Errorf("%s: command not found", job.Name)
 		job.status = 127
@@ -66,16 +90,22 @@ func (job *Job) Run() error {
 		job.status = job.handler(job)
 		job.end = time.Now()
 	}
-	// Wait for all background tasks to complete
-	if err := job.Stdout.Close(); err != nil {
-		return err
-	}
-	if err := job.Stderr.Close(); err != nil {
-		return err
+	if job.closeIO {
+		// Wait for all background tasks to complete
+		if err := job.Stdout.Close(); err != nil {
+			return err
+		}
+		if err := job.Stderr.Close(); err != nil {
+			return err
+		}
+		if err := job.Stdin.Close(); err != nil {
+			return err
+		}
 	}
 	if job.status != 0 {
-		return fmt.Errorf("%s", errorMessage)
+		return fmt.Errorf("%s", Tail(errorMessage, 1))
 	}
+
 	return nil
 }
 
@@ -122,6 +152,14 @@ func (job *Job) SetenvBool(key string, value bool) {
 	job.env.SetBool(key, value)
 }
 
+func (job *Job) GetenvTime(key string) (value time.Time, err error) {
+	return job.env.GetTime(key)
+}
+
+func (job *Job) SetenvTime(key string, value time.Time) {
+	job.env.SetTime(key, value)
+}
+
 func (job *Job) GetenvSubEnv(key string) *Env {
 	return job.env.GetSubEnv(key)
 }
@@ -189,11 +227,8 @@ func (job *Job) Environ() map[string]string {
 }
 
 func (job *Job) Logf(format string, args ...interface{}) (n int, err error) {
-	if os.Getenv("TEST") == "" {
-		prefixedFormat := fmt.Sprintf("[%s] %s\n", job, strings.TrimRight(format, "\n"))
-		return fmt.Fprintf(job.Stderr, prefixedFormat, args...)
-	}
-	return 0, nil
+	prefixedFormat := fmt.Sprintf("[%s] %s\n", job, strings.TrimRight(format, "\n"))
+	return fmt.Fprintf(job.Stderr, prefixedFormat, args...)
 }
 
 func (job *Job) Printf(format string, args ...interface{}) (n int, err error) {
@@ -212,3 +247,23 @@ func (job *Job) Error(err error) Status {
 	fmt.Fprintf(job.Stderr, "%s\n", err)
 	return StatusErr
 }
+
+func (job *Job) StatusCode() int {
+	return int(job.status)
+}
+
+func (job *Job) SetCloseIO(val bool) {
+	job.closeIO = val
+}
+
+// When called, causes the Job.WaitCancelled channel to unblock.
+func (job *Job) Cancel() {
+	job.cancelOnce.Do(func() {
+		close(job.cancelled)
+	})
+}
+
+// Returns a channel which is closed ("never blocks") when the job is cancelled.
+func (job *Job) WaitCancelled() <-chan struct{} {
+	return job.cancelled
+}
