@@ -1,7 +1,6 @@
 package main
 
 import (
-	"fmt"
 	"os/exec"
 	"strings"
 	"testing"
@@ -11,10 +10,8 @@ import (
 func TestSearchOnCentralRegistry(t *testing.T) {
 	searchCmd := exec.Command(dockerBinary, "search", "busybox")
 	out, exitCode, err := runCommandWithOutput(searchCmd)
-	errorOut(err, t, fmt.Sprintf("encountered error while searching: %v", err))
-
 	if err != nil || exitCode != 0 {
-		t.Fatal("failed to search on the central registry")
+		t.Fatalf("failed to search on the central registry: %s, %v", out, err)
 	}
 
 	if !strings.Contains(out, "Busybox base image.") {
