@@ -5,6 +5,7 @@ import (
 
 	"github.com/docker/docker/engine"
 	"github.com/docker/docker/nat"
+	"github.com/docker/docker/pkg/ulimit"
 	"github.com/docker/docker/utils"
 )
 
@@ -98,10 +99,19 @@ type RestartPolicy struct {
 	MaximumRetryCount int
 }
 
+type LogConfig struct {
+	Type   string
+	Config map[string]string
+}
+
 type HostConfig struct {
 	Binds           []string
 	ContainerIDFile string
 	LxcConf         []utils.KeyValuePair
+	Memory          int64  // Memory limit (in bytes)
+	MemorySwap      int64  // Total memory usage (memory + swap); set `-1` to disable swap
+	CpuShares       int64  // CPU shares (relative weight vs. other containers)
+	CpusetCpus      string // CpusetCpus 0-2, 0,1
 	Privileged      bool
 	PortBindings    nat.PortMap
 	Links           []string
@@ -119,6 +129,9 @@ type HostConfig struct {
 	RestartPolicy   RestartPolicy
 	SecurityOpt     []string
 	ReadonlyRootfs  bool
+	Ulimits         []*ulimit.Ulimit
+	LogConfig       LogConfig
+	CgroupParent    string // Parent cgroup.
 }
 
 // This is used by the create command when you want to set both the
@@ -139,23 +152,53 @@ func ContainerHostConfigFromJob(job *engine.Job) *HostConfig {
 	if job.EnvExists("HostConfig") {
 		hostConfig := HostConfig{}
 		job.GetenvJson("HostConfig", &hostConfig)
+
+		// FIXME: These are for backward compatibility, if people use these
+		// options with `HostConfig`, we should still make them workable.
+		if job.EnvExists("Memory") && hostConfig.Memory == 0 {
+			hostConfig.Memory = job.GetenvInt64("Memory")
+		}
+		if job.EnvExists("MemorySwap") && hostConfig.MemorySwap == 0 {
+			hostConfig.MemorySwap = job.GetenvInt64("MemorySwap")
+		}
+		if job.EnvExists("CpuShares") && hostConfig.CpuShares == 0 {
+			hostConfig.CpuShares = job.GetenvInt64("CpuShares")
+		}
+		if job.EnvExists("Cpuset") && hostConfig.CpusetCpus == "" {
+			hostConfig.CpusetCpus = job.Getenv("Cpuset")
+		}
+
 		return &hostConfig
 	}
 
 	hostConfig := &HostConfig{
 		ContainerIDFile: job.Getenv("ContainerIDFile"),
+		Memory:          job.GetenvInt64("Memory"),
+		MemorySwap:      job.GetenvInt64("MemorySwap"),
+		CpuShares:       job.GetenvInt64("CpuShares"),
+		CpusetCpus:      job.Getenv("CpusetCpus"),
 		Privileged:      job.GetenvBool("Privileged"),
 		PublishAllPorts: job.GetenvBool("PublishAllPorts"),
 		NetworkMode:     NetworkMode(job.Getenv("NetworkMode")),
 		IpcMode:         IpcMode(job.Getenv("IpcMode")),
 		PidMode:         PidMode(job.Getenv("PidMode")),
 		ReadonlyRootfs:  job.GetenvBool("ReadonlyRootfs"),
+		CgroupParent:    job.Getenv("CgroupParent"),
+	}
+
+	// FIXME: This is for backward compatibility, if people use `Cpuset`
+	// in json, make it workable, we will only pass hostConfig.CpusetCpus
+	// to execDriver.
+	if job.EnvExists("Cpuset") && hostConfig.CpusetCpus == "" {
+		hostConfig.CpusetCpus = job.Getenv("Cpuset")
 	}
 
 	job.GetenvJson("LxcConf", &hostConfig.LxcConf)
 	job.GetenvJson("PortBindings", &hostConfig.PortBindings)
 	job.GetenvJson("Devices", &hostConfig.Devices)
 	job.GetenvJson("RestartPolicy", &hostConfig.RestartPolicy)
+	job.GetenvJson("Ulimits", &hostConfig.Ulimits)
+	job.GetenvJson("LogConfig", &hostConfig.LogConfig)
 	hostConfig.SecurityOpt = job.GetenvList("SecurityOpt")
 	if Binds := job.GetenvList("Binds"); Binds != nil {
 		hostConfig.Binds = Binds
