@@ -3,15 +3,16 @@ package utils
 import (
 	"encoding/json"
 	"fmt"
+	"github.com/docker/docker/pkg/progressreader"
+	"io"
 )
 
 type StreamFormatter struct {
 	json bool
-	used bool
 }
 
 func NewStreamFormatter(json bool) *StreamFormatter {
-	return &StreamFormatter{json, false}
+	return &StreamFormatter{json}
 }
 
 const streamNewline = "\r\n"
@@ -19,7 +20,6 @@ const streamNewline = "\r\n"
 var streamNewlineBytes = []byte(streamNewline)
 
 func (sf *StreamFormatter) FormatStream(str string) []byte {
-	sf.used = true
 	if sf.json {
 		b, err := json.Marshal(&JSONMessage{Stream: str})
 		if err != nil {
@@ -31,7 +31,6 @@ func (sf *StreamFormatter) FormatStream(str string) []byte {
 }
 
 func (sf *StreamFormatter) FormatStatus(id, format string, a ...interface{}) []byte {
-	sf.used = true
 	str := fmt.Sprintf(format, a...)
 	if sf.json {
 		b, err := json.Marshal(&JSONMessage{ID: id, Status: str})
@@ -44,7 +43,6 @@ func (sf *StreamFormatter) FormatStatus(id, format string, a ...interface{}) []b
 }
 
 func (sf *StreamFormatter) FormatError(err error) []byte {
-	sf.used = true
 	if sf.json {
 		jsonError, ok := err.(*JSONError)
 		if !ok {
@@ -57,12 +55,19 @@ func (sf *StreamFormatter) FormatError(err error) []byte {
 	}
 	return []byte("Error: " + err.Error() + streamNewline)
 }
-
+func (sf *StreamFormatter) FormatProg(id, action string, p interface{}) []byte {
+	switch progress := p.(type) {
+	case *JSONProgress:
+		return sf.FormatProgress(id, action, progress)
+	case progressreader.PR_JSONProgress:
+		return sf.FormatProgress(id, action, &JSONProgress{Current: progress.GetCurrent(), Total: progress.GetTotal()})
+	}
+	return nil
+}
 func (sf *StreamFormatter) FormatProgress(id, action string, progress *JSONProgress) []byte {
 	if progress == nil {
 		progress = &JSONProgress{}
 	}
-	sf.used = true
 	if sf.json {
 
 		b, err := json.Marshal(&JSONMessage{
@@ -83,10 +88,34 @@ func (sf *StreamFormatter) FormatProgress(id, action string, progress *JSONProgr
 	return []byte(action + " " + progress.String() + endl)
 }
 
-func (sf *StreamFormatter) Used() bool {
-	return sf.used
-}
-
 func (sf *StreamFormatter) Json() bool {
 	return sf.json
 }
+
+type StdoutFormater struct {
+	io.Writer
+	*StreamFormatter
+}
+
+func (sf *StdoutFormater) Write(buf []byte) (int, error) {
+	formattedBuf := sf.StreamFormatter.FormatStream(string(buf))
+	n, err := sf.Writer.Write(formattedBuf)
+	if n != len(formattedBuf) {
+		return n, io.ErrShortWrite
+	}
+	return len(buf), err
+}
+
+type StderrFormater struct {
+	io.Writer
+	*StreamFormatter
+}
+
+func (sf *StderrFormater) Write(buf []byte) (int, error) {
+	formattedBuf := sf.StreamFormatter.FormatStream("\033[91m" + string(buf) + "\033[0m")
+	n, err := sf.Writer.Write(formattedBuf)
+	if n != len(formattedBuf) {
+		return n, io.ErrShortWrite
+	}
+	return len(buf), err
+}
